import asyncio
import aio_pika
import aiohttp
from loguru import logger
import os, sys
from collections import defaultdict
import time

# 环境变量或配置文件中获取敏感信息
rabbitmq_user = os.getenv("RABBITMQ_USER", "bot")
rabbitmq_password = os.getenv("RABBITMQ_PASSWORD", "xiaomi@123")
rabbitmq_host = os.getenv("RABBITMQ_HOST", "mq.stupidpz.com")
rabbitmq_port = int(os.getenv("RABBITMQ_PORT", 5672))

# 移除默认的日志处理器
logger.remove()

# 添加新的日志处理器，设置日志级别为 DEBUG
logger.add(sys.stderr, level="INFO")


class RateLimiter:
    def __init__(self, rate, per):
        self.rate = rate  # 短时间内允许的最大请求数
        self.per = per  # 时间窗口，单位秒
        self.allowance = rate
        self.last_check = time.time()

    def can_send(self):
        current = time.time()
        time_passed = current - self.last_check
        self.last_check = current
        self.allowance += time_passed * (self.rate / self.per)
        if self.allowance > self.rate:
            self.allowance = self.rate

        if self.allowance < 1.0:
            return False
        else:
            self.allowance -= 1.0
            return True


async def _send_message_to_user(bot_token, target_id, message, rate_limiter, session):
    if not rate_limiter.can_send():
        await asyncio.sleep(1)
        return await _send_message_to_user(bot_token, target_id, message, rate_limiter, session)

    parse_mode = determine_parse_mode(message)

    base_url = f"https://api.telegram.org/bot{bot_token}/sendMessage"
    params = {
        "chat_id": target_id,
        "text": message,
        "parse_mode": parse_mode
    }

    max_retry = 3
    retry_count = 0
    while retry_count < max_retry:
        try:
            async with session.post(base_url, params=params) as response:
                if response.status == 200:
                    logger.debug(f'消息发送成功: {message}')
                    return
                else:
                    logger.debug('消息发送失败，重试中...')
                    logger.error(f'{params},发送失败,返回值:{await response.text()}')
        except aiohttp.ClientError as e:
            logger.error(f'网络异常，重试中... 错误详情: {e}')
        await asyncio.sleep(10)
        retry_count += 1

    logger.debug('消息发送失败')


def determine_parse_mode(message_text):
    """
    Determine the appropriate parse mode based on the message content.
    For simplicity, assume that if the message contains certain HTML tags, use HTML parse mode.
    Otherwise, use MarkdownV2.
    """
    html_tags = ['<b>', '<i>', '<a>', '<code>', '<pre>']
    if any(tag in message_text for tag in html_tags):
        return 'HTML'
    return 'MarkdownV2'


def parse_message(body):
    parts = body.split('|')
    if len(parts) == 3:
        bot_token, target_id, message_text = parts
    else:
        parts = body.split('#')
        if len(parts) == 3:
            bot_token, target_id, message_text = parts
        else:
            raise ValueError("Unknown message format")
    return bot_token, target_id, message_text


async def message_handler(message: aio_pika.IncomingMessage):
    async with message.process():
        logger.info(message.body.decode())
        try:
            bot_token, target_id, message_text = parse_message(message.body.decode())
        except ValueError as e:
            logger.error(f'无法解析消息格式: {e}')
            return

        if target_id.startswith('-'):  # 群组ID以'-'开头
            rate_limiter = group_rate_limiters[bot_token]
        else:
            rate_limiter = rate_limiters[bot_token]
        if message_text:
            await _send_message_to_user(bot_token, target_id, message_text, rate_limiter, session)


async def main():
    connection = await aio_pika.connect_robust(
        host=rabbitmq_host,
        port=rabbitmq_port,
        login=rabbitmq_user,
        password=rabbitmq_password
    )

    channel = await connection.channel()
    queue = await channel.declare_queue('message_queue')

    global session
    async with aiohttp.ClientSession() as session:
        await queue.consume(message_handler)

        logger.info("开始消费消息...")
        try:
            await asyncio.Future()
        finally:
            await connection.close()


if __name__ == "__main__":
    rate_limiters = defaultdict(lambda: RateLimiter(rate=30, per=1))  # 每秒最多30条消息
    group_rate_limiters = defaultdict(lambda: RateLimiter(rate=20, per=60))  # 每分钟最多20条消息

    try:
        asyncio.run(main())
    except KeyboardInterrupt:
        logger.info("程序被手动停止")
